#include <iostream>
#include <string>

using namespace std;

class Thing {
	static unsigned lastId; // this is a declaration only!

	const unsigned id;
	const string name;

public:
	Thing(const string& name)
	    : id(lastId++)
	    , name(name) {}
	void tell() const { cout << id << ": " << name << '\n'; }

	static unsigned getThingsCount() { return lastId; }
};

// before C++17, static member variables have to be explicitly defined outside
// of the class (and also outside of any header files)
//
// since C++17, we can also define inline static member variables inside
// class declarations

unsigned Thing::lastId = 0;

int main() {
	Thing hammer("hammer");
	Thing pen("pen");
	Thing lte("laser-guided tactical eliminator");

	cout << Thing::getThingsCount() << '\n';

	for (const Thing& t : { hammer, pen, lte }) {
		t.tell();
	}
}
